import React, { PropTypes } from 'react'
import cx from 'classnames'
import { withTheme } from 'styled-components'
import {
  SpaceScale,
  SpaceScalePropType
} from './../components'

const SpacingPropType = PropTypes.oneOfType([
  PropTypes.bool,
  SpaceScalePropType
])

const parseUnit = (unit) => {
  if (typeof unit === 'boolean') {
    return SpaceScale[4]
  } else if (Number.isInteger(unit)) {
    return SpaceScale[unit]
  }
  return unit
}

const parseStyleProps = ({ all, top, right, bottom, left }) => {
  const pAll = parseUnit(all) || 0
  return `${parseUnit(top) || pAll} ${parseUnit(right) || pAll} ${parseUnit(bottom) || pAll} ${parseUnit(left) || pAll}`
}

const parseProps = ({
  padding,
  paddingTop,
  paddingRight,
  paddingBottom,
  paddingLeft,
  paddingX,
  paddingY,
  margin,
  marginTop,
  marginRight,
  marginBottom,
  marginLeft,
  marginX,
  marginY,
  style,
  ...props
}) => {
  const styles = {}

  if (padding !== undefined
  || paddingTop !== undefined
  || paddingRight !== undefined
  || paddingBottom !== undefined
  || paddingLeft !== undefined
  || paddingX !== undefined
  || paddingY !== undefined) {
    styles.padding = parseStyleProps({
      all: padding,
      top: paddingTop || paddingY,
      right: paddingRight || paddingX,
      bottom: paddingBottom || paddingY,
      left: paddingLeft || paddingX
    })
  }

  if (margin !== undefined
   || marginTop !== undefined
   || marginRight !== undefined
   || marginBottom !== undefined
   || marginLeft !== undefined
   || marginX !== undefined
   || marginY !== undefined) {
    styles.margin = parseStyleProps({
      all: margin,
      top: marginTop || marginY,
      right: marginRight || marginX,
      bottom: marginBottom || marginY,
      left: marginLeft || marginX
    })
  }

  return { style: { ...styles, ...style }, ...props }
}

const withSpacing = (Comp) => {
  const SpacedComp = (originalProps) => {
    return React.createElement(Comp, parseProps(originalProps))
  }

  SpacedComp.propTypes = {
    ...Comp.propTypes,
    padding: SpacingPropType,
    paddingTop: SpacingPropType,
    paddingRight: SpacingPropType,
    paddingBottom: SpacingPropType,
    paddingLeft: SpacingPropType,
    paddingX: SpacingPropType,
    paddingY: SpacingPropType,
    margin: SpacingPropType,
    marginTop: SpacingPropType,
    marginRight: SpacingPropType,
    marginBottom: SpacingPropType,
    marginLeft: SpacingPropType,
    marginX: SpacingPropType,
    marginY: SpacingPropType
  }

  SpacedComp.displayName = Comp.displayName

  return withTheme(SpacedComp)
}

export {
  withSpacing
}
